// -*- Mode: C++; tab-width: 2; -*-
// vi: set ts=2:
//
// --------------------------------------------------------------------------
//                   OpenMS Mass Spectrometry Framework
// --------------------------------------------------------------------------
//  Copyright (C) 2003-2008 -- Oliver Kohlbacher, Knut Reinert
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// --------------------------------------------------------------------------
// $Maintainer: Andreas Bertsch $
// --------------------------------------------------------------------------
//
#ifndef OPENMS_FILTERING_TRANSFORMERS_SCALER_H
#define OPENMS_FILTERING_TRANSFORMERS_SCALER_H

#include <OpenMS/FILTERING/TRANSFORMERS/PreprocessingFunctor.h>
#include <map>

namespace OpenMS
{
  /**
  	@brief Scaler scales the peak by ranking the peaks and assigning intensity according to rank

		@ingroup SpectraPreprocessers
  */
  class Scaler
    : public PreprocessingFunctor
  {
  public:

		// @name Constructors and Destructors
		// @{
    /// default constructor
    Scaler();

    /// copy constructor
    Scaler(const Scaler& source);

    /// destructor
    virtual ~Scaler();
		// @}

		// @name Operators
		// @{
    /// assignment operator
    Scaler& operator = (const Scaler& source);
		// @}

		// @name Accessors
		// @{
		///
    static PreprocessingFunctor* create() { return new Scaler(); }

		///
		static const String getProductName()
		{
			return "Scaler";
		}

		template <typename SpectrumType> void filterSpectrum(SpectrumType& spectrum)
		{	
			if (spectrum.size()==0) return;
			
			spectrum.getContainer().sortByIntensity();
			typename SpectrumType::size_type count = spectrum.size();
			++count;
			typename SpectrumType::PeakType::IntensityType last_int = 0.0; 
			typename SpectrumType::Iterator it = spectrum.end();
			do
			{
				--it;
				if (it->getIntensity()!=last_int)
				{
					--count;
				}
				last_int = it->getIntensity();
				it->setIntensity(count);
			}
			while (it!=spectrum.begin());
		}

		void filterPeakSpectrum(PeakSpectrum& spectrum);

    void filterPeakMap(PeakMap& exp);
		// @}

  };

}
#endif //OPENMS_FILTERING_TRANSFORMERS_SCALER_H
