// -*- Mode: C++; tab-width: 2; -*-
// vi: set ts=2:
//
// --------------------------------------------------------------------------
//                   OpenMS Mass Spectrometry Framework 
// --------------------------------------------------------------------------
//  Copyright (C) 2003-2008 -- Oliver Kohlbacher, Knut Reinert
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// --------------------------------------------------------------------------
// $Maintainer: Marc Sturm $
// --------------------------------------------------------------------------

#include <OpenMS/CONCEPT/ClassTest.h>

///////////////////////////

#include <OpenMS/FILTERING/DATAREDUCTION/DataFilters.h>
#include <OpenMS/KERNEL/Feature.h>
#include <OpenMS/KERNEL/Peak1D.h>

///////////////////////////

START_TEST(DataFilters, "$Id: DataFilters_test.C 2783 2008-02-28 16:18:55Z marc_sturm $")

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

using namespace OpenMS;
using namespace std;
	
///constructor and destructor test
DataFilters* ptr;
CHECK((DataFilters()))
	ptr = new DataFilters();
	TEST_NOT_EQUAL(ptr, 0)
RESULT

CHECK((~DataFilters()))
	delete ptr;	
RESULT

DataFilters::DataFilter* ptr2;
CHECK((DataFilters::DataFilter()))
	ptr2 = new DataFilters::DataFilter();
	TEST_NOT_EQUAL(ptr2, 0)
RESULT

CHECK((~DataFilters::DataFilter()))
	delete ptr2;	
RESULT


DataFilters::DataFilter filter_1;
DataFilters::DataFilter filter_2;
DataFilters::DataFilter filter_3;
DataFilters::DataFilter filter_4;
DataFilters::DataFilter filter_5;
DataFilters::DataFilter filter_6;
DataFilters::DataFilter filter_7;
DataFilters::DataFilter filter_8;
DataFilters::DataFilter filter_9;
DataFilters::DataFilter filter_10;
DataFilters::DataFilter filter_11;


CHECK ((void DataFilter::fromString(const String& filter) throw (Exception::InvalidValue)))

	TEST_EXCEPTION_WITH_MESSAGE(Exception::InvalidValue, filter_1.fromString(""), "The value `' was used but is not valid! Invalid filter format.")
	TEST_EXCEPTION_WITH_MESSAGE(Exception::InvalidValue, filter_1.fromString("not_enough_arguments"), "The value `not_enough_arguments' was used but is not valid! Invalid filter format.")
	TEST_EXCEPTION_WITH_MESSAGE(Exception::InvalidValue, filter_1.fromString("invalid_fieldname = 0"), "The value `invalid_fieldname' was used but is not valid! Invalid field name.")
	TEST_EXCEPTION_WITH_MESSAGE(Exception::InvalidValue, filter_1.fromString("Intensity invalid_operator 5"), "The value `invalid_operator' was used but is not valid! Invalid operator.")
	TEST_EXCEPTION_WITH_MESSAGE(Exception::InvalidValue, filter_1.fromString("Meta::test = string without enclosing quotation marks"), "The value `string without enclosing quotation marks' was used but is not valid! Invalid value.")
	//second argument of binary relation missing:
	TEST_EXCEPTION_WITH_MESSAGE(Exception::InvalidValue, filter_1.fromString("Charge = "), "The value `=' was used but is not valid! Invalid filter format.")
	//string value and non-meta field:
	TEST_EXCEPTION_WITH_MESSAGE(Exception::InvalidValue, filter_1.fromString("Quality = \"a string\""), "The value `a string' was used but is not valid! Invalid value.")
	//operation "exists" and non-meta field:
	TEST_EXCEPTION_WITH_MESSAGE(Exception::InvalidValue, filter_1.fromString("Intensity exists"), "The value `exists' was used but is not valid! Invalid operator.")
	
	filter_1.fromString("Intensity <= 201.334");
	filter_2.fromString("Intensity >= 1000");
	filter_3.fromString("Charge = 4");
	filter_4.fromString("Quality <= 1.0");
	filter_5.fromString("Meta::test_int <= 0");
	filter_6.fromString("Meta::test_double = 0");
	filter_7.fromString("Meta::test_string = \"hello world 2\"");
	filter_8.fromString("Meta::test_dummy exists");
	//valid, but nonsense (nothing will pass this filter):
	filter_9.fromString("Meta::test_string >= \"a string\"");
	filter_10.fromString("Meta::test_string = \"hello world 2\"");
	filter_11.fromString("Meta::unknown_metavalue = 5");
	
RESULT


CHECK ((String DataFilter::toString() const))
	
	TEST_STRING_EQUAL(filter_1.toString(), "Intensity <= 201.334")
	TEST_STRING_EQUAL(filter_2.toString(), "Intensity >= 1000")
	TEST_STRING_EQUAL(filter_3.toString(), "Charge = 4")
	TEST_STRING_EQUAL(filter_4.toString(), "Quality <= 1")
	TEST_STRING_EQUAL(filter_5.toString(), "Meta::test_int <= 0")
	TEST_STRING_EQUAL(filter_6.toString(), "Meta::test_double = 0")
	TEST_STRING_EQUAL(filter_7.toString(), "Meta::test_string = \"hello world 2\"")
	TEST_STRING_EQUAL(filter_8.toString(), "Meta::test_dummy exists")
	TEST_STRING_EQUAL(filter_9.toString(), "Meta::test_string >= \"a string\"")

RESULT


CHECK ((bool DataFilter::operator==(const DataFilter& rhs) const))

	TEST_EQUAL(filter_10 == filter_7, true)
	TEST_EQUAL(filter_1 == filter_2, false)
	TEST_EQUAL(filter_3 == filter_3, true)

RESULT


CHECK ((bool DataFilter::operator!=(const DataFilter& rhs) const))

	TEST_EQUAL(filter_10 != filter_7, false)
	TEST_EQUAL(filter_3 != filter_4, true)
	TEST_EQUAL(filter_4 != filter_4, false)
	
RESULT


DataFilters filters;

CHECK ((void add(const DataFilter& filter)))

	filters.add(filter_1);
	filters.add(filter_2);
	filters.add(filter_3);
	
	TEST_EQUAL(filters[0] == filter_1, true)
	TEST_EQUAL(filters[1] == filter_2, true)
	TEST_EQUAL(filters[2] == filter_3, true)
	
RESULT


CHECK ((const DataFilter& operator[](UInt index) const throw (Exception::IndexOverflow)))
	
	TEST_EXCEPTION(Exception::IndexOverflow, filters[3])
	filters.add(filter_1);
	TEST_EQUAL(filters[0] == filters[3], true)
	filters.remove(3);
	
RESULT


CHECK ((UInt size() const))

	TEST_EQUAL(filters.size(), 3)
	filters.add(filter_4);
	TEST_EQUAL(filters.size(), 4)
	filters.add(filter_5);
	filters.add(filter_6);
	filters.add(filter_7);
	filters.add(filter_8);
	filters.add(filter_9);
	TEST_EQUAL(filters.size(), 9)
	filters.remove(0);
	TEST_EQUAL(filters.size(), 8)
	filters.remove(0);
	TEST_EQUAL(filters.size(), 7)

RESULT


CHECK ((void remove(UInt index) throw (Exception::IndexOverflow)))

	TEST_EXCEPTION(Exception::IndexOverflow, filters.remove(7))
	filters.remove(0);
	TEST_EQUAL(filters[0] == filter_4, true)
	filters.remove(0);
	TEST_EQUAL(filters[0] == filter_5, true)
	
RESULT


CHECK ((void replace(UInt index, const DataFilter& filter) throw (Exception::IndexOverflow)))
	
	TEST_EXCEPTION(Exception::IndexOverflow, filters.replace(10, filter_1))
	//at the moment: filters[0] == filter_5, ..., filters[4] == filter_9
	filters.replace(0, filter_1);
	filters.replace(1, filter_2);
	filters.replace(2, filter_3);
	filters.replace(3, filter_4);
	filters.replace(4, filter_5);
	TEST_EQUAL(filters[0] == filter_1, true)
	TEST_EQUAL(filters[1] == filter_2, true)
	TEST_EQUAL(filters[2] == filter_3, true)
	TEST_EQUAL(filters[3] == filter_4, true)
	TEST_EQUAL(filters[4] == filter_5, true)
	TEST_EQUAL(filters.size(), 5)
	
RESULT


CHECK ((void clear()))
	
	filters.clear();
	TEST_EQUAL(filters.size(), 0)

RESULT


///construct some test features
Feature feature_1;
feature_1.setIntensity(1000.00);
feature_1.setCharge(4);
feature_1.setOverallQuality(31.3334);
feature_1.setMetaValue(String("test_int"), 5);
feature_1.setMetaValue(String("test_double"), 23.42);
feature_1.setMetaValue(String("test_string"), String("hello world 1"));

Feature feature_2;
feature_2.setIntensity(122.01);
feature_2.setCharge(3);
feature_2.setOverallQuality(0.002);
feature_2.setMetaValue(String("test_int"), 10);
feature_2.setMetaValue(String("test_double"), 0.042);
feature_2.setMetaValue(String("test_string"), String("hello world 2"));

Feature feature_3;
feature_3.setIntensity(55.0);
feature_3.setCharge(4);
feature_3.setOverallQuality(1);
feature_3.setMetaValue(String("test_int"), 0);
feature_3.setMetaValue(String("test_double"), 100.01);
feature_3.setMetaValue(String("test_string"), String("hello world 3"));

///and some test peaks
Peak1D peak_1;
peak_1.setIntensity(201.334);
peak_1.setMetaValue(String("test_int"), 5);
peak_1.setMetaValue(String("test_double"), 23.42);
peak_1.setMetaValue(String("test_string"), String("hello world 1"));
peak_1.setMetaValue(String("test_dummy"), 10);

Peak1D peak_2;
peak_2.setIntensity(2008.2);
peak_2.setMetaValue(String("test_int"), 10);
peak_2.setMetaValue(String("test_double"), 0.000);
peak_2.setMetaValue(String("test_string"), String("hello world 2"));

Peak1D peak_3;
peak_3.setIntensity(0.001);
peak_3.setMetaValue(String("test_int"), 0);
peak_3.setMetaValue(String("test_double"), 100.01);
peak_3.setMetaValue(String("test_string"), String("hello world 3"));


CHECK ((template<class PeakType> bool passes(const PeakType& peak) const))

	filters.add(filter_1); // "Intensity <= 201.334"
	TEST_EQUAL(filters.passes(peak_1), true) // 201.334
	TEST_EQUAL(filters.passes(peak_2), false) // 2008.2
	TEST_EQUAL(filters.passes(peak_3), true) // 0.001
	
	filters.add(filter_2); // "Intensity <= 201.334" && "Intensity >= 1000"
	TEST_EQUAL(filters.passes(peak_1), false) // 201.334
	TEST_EQUAL(filters.passes(peak_2), false) // 2008.2
	TEST_EQUAL(filters.passes(peak_3), false) // 0.001
	
	filters.remove(0); // "Intensity >= 1000"
	TEST_EQUAL(filters.passes(peak_1), false) // 201.334
	TEST_EQUAL(filters.passes(peak_2), true) // 2008.2
	TEST_EQUAL(filters.passes(peak_3), false) // 0.001
	
	filters.clear();
	filters.add(filter_5); // "Meta::test_int <= 0"
	TEST_EQUAL(filters.passes(peak_1), false) // 5
	TEST_EQUAL(filters.passes(peak_2), false) // 10
	TEST_EQUAL(filters.passes(peak_3), true) // 0
	
	filters.clear();
	filters.add(filter_6); // Meta::test_double = 0
	filters.add(filter_7); // Meta::test_string = "hello world 2"
	TEST_EQUAL(filters.passes(peak_1), false) // test_double = 23.42; test_string = "hello world 1"
	TEST_EQUAL(filters.passes(peak_2), true) // test_double = 0.000; test_string = "hello world 2"
	TEST_EQUAL(filters.passes(peak_3), false) // test_double = 100.01; test_string = "hello world 3"
	
	filters.clear();
	filters.add(filter_8); // Meta::test_dummy exists
	TEST_EQUAL(filters.passes(peak_1), true) // exists here
	TEST_EQUAL(filters.passes(peak_2), false)
	TEST_EQUAL(filters.passes(peak_3), false)
		
	filters.clear();
	filters.add(filter_9); // "Meta::test_string >= \"a string\"" (nonsense)
	TEST_EQUAL(filters.passes(peak_1), false)
	TEST_EQUAL(filters.passes(peak_2), false)
	TEST_EQUAL(filters.passes(peak_3), false)
	
	filters.clear();
	filters.add(filter_11); // "Meta::unknown_metavalue = 5" (if meta value is not set, peak will not pass)
	TEST_EQUAL(filters.passes(peak_1), false)
	TEST_EQUAL(filters.passes(peak_2), false)
	TEST_EQUAL(filters.passes(peak_3), false)
	
RESULT


CHECK ((bool passes(const Feature& feature) const))

	filters.clear();
	filters.add(filter_3); // "Charge = 4"
	TEST_EQUAL(filters.passes(feature_1), true) // 4
	TEST_EQUAL(filters.passes(feature_2), false) // 3
	TEST_EQUAL(filters.passes(feature_3), true) // 4
	
	filters.add(filter_4); // "Quality <= 1.0" && "Charge = 4"
	TEST_EQUAL(filters.passes(feature_1), false) // Quality = 31.3334; Charge = 4
	TEST_EQUAL(filters.passes(feature_2), false) // Quality = 0.002; Charge = 3
	TEST_EQUAL(filters.passes(feature_3), true) // Quality = 1; Charge = 4
	
	filters.remove(0); // "Quality <= 1.0"
	TEST_EQUAL(filters.passes(feature_1), false) // Quality = 31.3334
	TEST_EQUAL(filters.passes(feature_2), true) // Quality = 0.002
	TEST_EQUAL(filters.passes(feature_3), true) // Quality = 1
	
	filters.clear();
	filters.add(filter_2); // "Intensity >= 1000"
	TEST_EQUAL(filters.passes(feature_1), true) // 1000.00
	TEST_EQUAL(filters.passes(feature_2), false) // 122.01
	TEST_EQUAL(filters.passes(feature_3), false) // 55.0
	
	filters.clear();
	filters.add(filter_7); // "Meta::test_string = \"hello world 2\""
	TEST_EQUAL(filters.passes(feature_1), false)
	TEST_EQUAL(filters.passes(feature_2), true)
	TEST_EQUAL(filters.passes(feature_3), false)
	
	filters.add(filter_8); // "Meta::test_dummy exists"
	TEST_EQUAL(filters.passes(feature_1), false)
	TEST_EQUAL(filters.passes(feature_2), false)
	TEST_EQUAL(filters.passes(feature_3), false)
	
	filters.clear();
	filters.add(filter_5); // "Meta::test_int <= 0"
	TEST_EQUAL(filters.passes(feature_1), false) // 5
	TEST_EQUAL(filters.passes(feature_2), false) // 10
	TEST_EQUAL(filters.passes(feature_3), true) // 0

RESULT


/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

END_TEST
