// -*- Mode: C++; tab-width: 2; -*-
// vi: set ts=2:
//
// --------------------------------------------------------------------------
//                   OpenMS Mass Spectrometry Framework
// --------------------------------------------------------------------------
//  Copyright (C) 2003-2008 -- Oliver Kohlbacher, Knut Reinert
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// --------------------------------------------------------------------------
// $Maintainer: Clemens Groepl$
// --------------------------------------------------------------------------

#include <OpenMS/CONCEPT/ClassTest.h>

///////////////////////////
#include <OpenMS/TRANSFORMATIONS/FEATUREFINDER/FeatureFinderAlgorithm_impl.h>

///////////////////////////

using namespace OpenMS;
using namespace std;

namespace OpenMS
{
	template <class PeakType, class FeatureType>
	class FFA
		:public FeatureFinderAlgorithm<PeakType,FeatureType>
	{
		public:
			FFA()
				: FeatureFinderAlgorithm<PeakType,FeatureType>()
			{
			}
			
			~FFA()
			{
			}
			
			virtual void run()
			{
				
			}
			
			virtual Param getDefaultParameters() const
			{
				Param tmp;
				tmp.setValue("bla","bluff");
				return tmp;
			}
			
			const MSExperiment<PeakType>* getMap()
			{
				return this->map_;
			}
		
			const FeatureMap<Feature>* getFeatures()
			{
				return this->features_;
			}
		
			const FeatureFinder* getFF()
			{
				return this->ff_;
			}
	};
}

START_TEST(FeatureFinder, "$Id FeatureFinder_test.C 139 2006-07-14 10:08:39Z ole_st $")

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

FFA<RawDataPoint1D,Feature>* ptr = 0;
CHECK((FeatureFinderAlgorithm()))
	ptr = new FFA<RawDataPoint1D,Feature>();
	TEST_NOT_EQUAL(ptr, 0)
RESULT

CHECK((virtual ~FeatureFinderAlgorithm()))
	delete ptr;
RESULT

CHECK(virtual void run())
	FFA<RawDataPoint1D,Feature> ffa;
	ffa.run();
RESULT

CHECK(virtual Param getDefaultParameters() const)
	FFA<RawDataPoint1D,Feature> ffa;
	TEST_EQUAL(String(ffa.getDefaultParameters().getValue("bla")),"bluff")
RESULT

CHECK(void setData(const MapType& map, FeatureMapType& features, FeatureFinder& ff))
	FFA<RawDataPoint1D,Feature> ffa;
	TEST_EQUAL(ffa.getMap(),0)
	TEST_EQUAL(ffa.getFeatures(),0)
	TEST_EQUAL(ffa.getFF(),0)
	
	MSExperiment<RawDataPoint1D> map;
	FeatureMap<Feature> features;
	FeatureFinder ff;
	ffa.setData(map, features, ff);

	TEST_NOT_EQUAL(ffa.getMap(),0)
	TEST_NOT_EQUAL(ffa.getFeatures(),0)
	TEST_NOT_EQUAL(ffa.getFF(),0)	
RESULT
		

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
END_TEST



