// -*- Mode: C++; tab-width: 2; -*-
// vi: set ts=2:
//
// --------------------------------------------------------------------------
//                   OpenMS Mass Spectrometry Framework 
// --------------------------------------------------------------------------
//  Copyright (C) 2003-2008 -- Oliver Kohlbacher, Knut Reinert
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// --------------------------------------------------------------------------
// $Maintainer: Marc Sturm $
// --------------------------------------------------------------------------

#include <OpenMS/CONCEPT/ClassTest.h>

///////////////////////////

#include <OpenMS/KERNEL/Peak1D.h>

///////////////////////////

START_TEST(Peak1D<D>, "$Id: Peak1D_test.C 2734 2008-02-12 12:01:25Z marc_sturm $")

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

using namespace OpenMS;

Peak1D* d10_ptr = 0;
CHECK((Peak1D()))
	d10_ptr = new Peak1D;
	TEST_NOT_EQUAL(d10_ptr, 0)
RESULT

CHECK((~Peak1D()))
	delete d10_ptr;
RESULT

CHECK((Peak1D(const Peak1D &p)))
	Peak1D p;
	p.setIntensity(123.456);
	p.setMetaValue("cluster_id",4711);
	
	Peak1D copy_of_p(p);

	TEST_REAL_EQUAL(copy_of_p.getIntensity(), 123.456)
	TEST_EQUAL(copy_of_p.getMetaValue("cluster_id"),DataValue(4711));
RESULT

CHECK((Peak1D& operator=(const Peak1D &rhs)))
	Peak1D p;
	p.setIntensity(123.456);
	p.setMetaValue("cluster_id",4711);
	
	Peak1D copy_of_p;
	copy_of_p = p;

	TEST_REAL_EQUAL(copy_of_p.getIntensity(), 123.456)
	TEST_EQUAL(copy_of_p.getMetaValue("cluster_id"),DataValue(4711));
RESULT

CHECK((bool operator == (const Peak1D& rhs) const))
	Peak1D p1, p2;
	TEST_REAL_EQUAL(p1==p2, true)
	
	p1.setIntensity(5);
	TEST_REAL_EQUAL(p1==p2, false)
	p2.setIntensity(5);
	TEST_REAL_EQUAL(p1==p2, true)

	p1.setMetaValue("cluster_id",4711);
	TEST_REAL_EQUAL(p1==p2, false)
	p1.removeMetaValue("cluster_id");
	TEST_REAL_EQUAL(p1==p2, true)		
RESULT

CHECK((bool operator != (const Peak1D& rhs) const))
	Peak1D p1, p2;
	TEST_REAL_EQUAL(p1!=p2, false)
	
	p1.setIntensity(5);
	TEST_REAL_EQUAL(p1!=p2, true)
	p2.setIntensity(5);
	TEST_REAL_EQUAL(p1!=p2, false)

	p1.setMetaValue("cluster_id",4711);
	TEST_REAL_EQUAL(p1!=p2, true)
	p1.removeMetaValue("cluster_id");
	TEST_REAL_EQUAL(p1!=p2, false)	
RESULT

CHECK(([EXTRA] meta info with copy constructor))
	Peak1D p;
	p.setMetaValue(2,String("bla"));
 	Peak1D p2(p);
	TEST_EQUAL(p.getMetaValue(2), "bla")
	TEST_EQUAL(p2.getMetaValue(2), "bla")
 	p.setMetaValue(2,String("bluff"));
	TEST_EQUAL(p.getMetaValue(2), "bluff")
	TEST_EQUAL(p2.getMetaValue(2), "bla")
RESULT

CHECK(([EXTRA] meta info with assignment))
	Peak1D p;
	p.setMetaValue(2,String("bla"));
 	Peak1D p2 = p;
	TEST_EQUAL(p.getMetaValue(2), "bla")
	TEST_EQUAL(p2.getMetaValue(2), "bla")
 	p.setMetaValue(2,String("bluff"));
	TEST_EQUAL(p.getMetaValue(2), "bluff")
	TEST_EQUAL(p2.getMetaValue(2), "bla")
RESULT

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
END_TEST
