// -*- Mode: C++; tab-width: 2; -*-
// vi: set ts=2:
//
// --------------------------------------------------------------------------
//                   OpenMS Mass Spectrometry Framework
// --------------------------------------------------------------------------
//  Copyright (C) 2003-2008 -- Oliver Kohlbacher, Knut Reinert
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// --------------------------------------------------------------------------
// $Maintainer: Eva Lange $
// --------------------------------------------------------------------------

#include <OpenMS/CONCEPT/ClassTest.h>
#include <OpenMS/KERNEL/StandardTypes.h>

///////////////////////////
#include <OpenMS/ANALYSIS/MAPMATCHING/PoseClusteringAffineSuperimposer.h>
///////////////////////////

using namespace OpenMS;
using namespace std;

typedef DPosition <2> PositionType;

START_TEST(PoseClusteringAffineSuperimposer, "$Id: PoseClusteringAffineSuperimposer_test.C 2734 2008-02-12 12:01:25Z marc_sturm $")

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

PoseClusteringAffineSuperimposer<FeatureMap<> >* ptr = 0;
CHECK((PoseClusteringAffineSuperimposer()))
	ptr = new PoseClusteringAffineSuperimposer<FeatureMap<> >();
	TEST_NOT_EQUAL(ptr, 0)
RESULT

CHECK((virtual ~PoseClusteringAffineSuperimposer()))
	delete ptr;
RESULT

CHECK((static BaseSuperimposer<PointMapType>* create()))
  
RESULT

CHECK((static const String getProductName()))
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat;
  
  TEST_EQUAL(pcat.getName() == "poseclustering_affine",true)
RESULT

CHECK((virtual void run()))
  FeatureMap<> scene;
  Feature feat1;
  Feature feat2;
  PositionType pos1(1,1);
  PositionType pos2(5,5);
  feat1.setPosition(pos1);
  feat1.setIntensity(100);
  feat2.setPosition(pos2);
  feat2.setIntensity(100);
  scene.push_back(feat1);
  scene.push_back(feat2);
  
  FeatureMap<> modell;
  Feature feat3;
  Feature feat4;
  PositionType pos3(1.4,1.02);
  PositionType pos4(5.4,5.02);
  feat3.setPosition(pos3);
  feat3.setIntensity(100);
  feat4.setPosition(pos4);
  feat4.setIntensity(100);
  modell.push_back(feat3);
  modell.push_back(feat4);
  
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat;
  pcat.setElementMap(0,modell);
  pcat.setElementMap(1,scene);
  pcat.setScalingBucketSize(0,0.01);
  pcat.setScalingBucketSize(1,0.01);
  pcat.setShiftBucketSize(0,0.01);
  pcat.setShiftBucketSize(1,0.01);
  pcat.setBucketWindowScaling(0,0);
  pcat.setBucketWindowScaling(1,0);
  pcat.run();
    
  LinearMapping rt_mapping = pcat.getTransformation(0);
  LinearMapping mz_mapping = pcat.getTransformation(1);
  
  TEST_REAL_EQUAL(rt_mapping.getSlope(),1.0)
  TEST_REAL_EQUAL(rt_mapping.getIntercept(),0.4)
  TEST_REAL_EQUAL(mz_mapping.getSlope(),1)
  TEST_REAL_EQUAL(mz_mapping.getIntercept(),0.02)
RESULT

CHECK((UInt getBucketWindowScaling(UInt dim) const))
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat_copy;
  
  TEST_REAL_EQUAL(pcat_copy.getBucketWindowScaling(0),2)
  TEST_REAL_EQUAL(pcat_copy.getBucketWindowScaling(1),2)
RESULT

CHECK((UInt getBucketWindowShift(UInt dim) const))
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat_copy;
  
  TEST_REAL_EQUAL(pcat_copy.getBucketWindowShift(0),2)
  TEST_REAL_EQUAL(pcat_copy.getBucketWindowShift(1),2)
RESULT

CHECK((double getMzBucketSize() const))
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat_copy;
  
  TEST_REAL_EQUAL(pcat_copy.getMzBucketSize(),.5)
RESULT

CHECK((double getScalingBucketSize(UInt dim) const))
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat_copy;
  
  TEST_REAL_EQUAL(pcat_copy.getScalingBucketSize(0),0.01)
  TEST_REAL_EQUAL(pcat_copy.getScalingBucketSize(1),0.01)
RESULT

CHECK((double getShiftBucketSize(UInt dim) const))
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat_copy;
  
  TEST_REAL_EQUAL(pcat_copy.getShiftBucketSize(0),10)
  TEST_REAL_EQUAL(pcat_copy.getShiftBucketSize(1),0.01)
RESULT

CHECK((void setBucketWindowScaling(UInt dim, UInt bucket_window_scaling)))
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat;
  
  pcat.setBucketWindowScaling(0,6);
  pcat.setBucketWindowScaling(1,6);
  
  TEST_REAL_EQUAL(pcat.getBucketWindowScaling(0),6)
  TEST_REAL_EQUAL(pcat.getBucketWindowScaling(1),6)
RESULT

CHECK((void setBucketWindowShift(UInt dim, UInt bucket_window_shift)))
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat;
  
  pcat.setBucketWindowShift(0,2);
  pcat.setBucketWindowShift(1,4);
  
  TEST_REAL_EQUAL(pcat.getBucketWindowShift(0),2)
  TEST_REAL_EQUAL(pcat.getBucketWindowShift(1),4)
RESULT

CHECK((void setMzBucketSize(double mz_bucket_size)))
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat;
  
  pcat.setMzBucketSize(0.3);
  
  TEST_REAL_EQUAL(pcat.getMzBucketSize(),0.3)
RESULT

CHECK((void setScalingBucketSize(UInt dim, double scaling_bucket_size)))
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat;
  
  pcat.setScalingBucketSize(0,2.4);
  pcat.setScalingBucketSize(1,5.4);
  
  TEST_REAL_EQUAL(pcat.getScalingBucketSize(0),2.4)
  TEST_REAL_EQUAL(pcat.getScalingBucketSize(1),5.4)
RESULT

CHECK((void setShiftBucketSize(UInt dim, double shift_bucket_size)))
  PoseClusteringAffineSuperimposer<FeatureMap<> > pcat;
  
  pcat.setShiftBucketSize(0,0.3);
  pcat.setShiftBucketSize(1,1.4);
  
  TEST_REAL_EQUAL(pcat.getShiftBucketSize(0),0.3)
  TEST_REAL_EQUAL(pcat.getShiftBucketSize(1),1.4)
RESULT

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
END_TEST



