// -*- Mode: C++; tab-width: 2; -*-
// vi: set ts=2:
//
// --------------------------------------------------------------------------
//                   OpenMS Mass Spectrometry Framework 
// --------------------------------------------------------------------------
//  Copyright (C) 2003-2008 -- Oliver Kohlbacher, Knut Reinert
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// --------------------------------------------------------------------------
// $Maintainer: Marcel Grunert, Clemens Groepl$
// --------------------------------------------------------------------------

#include <OpenMS/CONCEPT/ClassTest.h>

///////////////////////////

#include <OpenMS/TRANSFORMATIONS/FEATUREFINDER/SimpleSeeder.h>
#include <OpenMS/TRANSFORMATIONS/FEATUREFINDER/FeatureFinder_impl.h>
///////////////////////////

START_TEST(SimpleSeeder, "$Id: SimpleSeeder_test.C 2734 2008-02-12 12:01:25Z marc_sturm $")

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

using namespace OpenMS;
using namespace std;

// default ctor
SimpleSeeder<RawDataPoint1D,Feature>* ptr = 0;
CHECK(SimpleSeeder(const MSExperiment<PeakType>* map, FeatureMap<FeatureType>* features, FeatureFinder* ff))
	MSExperiment<RawDataPoint1D> exp;
	ptr = new SimpleSeeder<RawDataPoint1D,Feature>(&exp,0,0);
  TEST_NOT_EQUAL(ptr, 0)
RESULT

CHECK(virtual ~SimpleSeeder())
	delete ptr;
RESULT


CHECK(IndexPair nextSeed() throw (NoSuccessor))

	//create map
	MSExperiment<RawDataPoint1D> exp;
	MSExperiment<RawDataPoint1D>::PeakType p;
	
	exp.resize(1);
	exp.back().setRT(1.0);
	p.setMZ(500.0);
	p.setIntensity(10.0);
	exp.back().push_back(p);
	p.setMZ(600.0);
	p.setIntensity(20.0);
	exp.back().push_back(p);
	p.setMZ(800.0);
	p.setIntensity(30.0);
	exp.back().push_back(p);
	p.setMZ(1000.0);
	p.setIntensity(40.0);
	exp.back().push_back(p);
	p.setMZ(1200.0);
	p.setIntensity(110.0);
	exp.back().push_back(p);
	
	exp.resize(2);
	exp.back().setRT(2.0);
	p.setMZ(500.0);
	p.setIntensity(101.0);
	exp.back().push_back(p);
	p.setMZ(600.0);
	p.setIntensity(81.0);
	exp.back().push_back(p);
	p.setMZ(800.0);
	p.setIntensity(31.0);
	exp.back().push_back(p);
	p.setMZ(1000.0);
	p.setIntensity(11.0);
	exp.back().push_back(p);
	p.setMZ(1200.0);
	p.setIntensity(111.0);
	exp.back().push_back(p);
	
	exp.updateRanges();
	
	//create dummy FF instance (for flags)
  FeatureFinder ff;
  FeatureMap<Feature> features;
  ff.run("none", exp,features, Param());
  
  //make two peaks used
	ff.getPeakFlag(make_pair(0,4)) = FeatureFinderDefs::USED;
	ff.getPeakFlag(make_pair(1,4)) = FeatureFinderDefs::USED;
	
	//First test (3 unused peaks with intensity greater than 35)
	{
		SimpleSeeder<RawDataPoint1D,Feature> seeder(&exp, &features, &ff);
		Param param;
		param.setValue("min_intensity",35.0);
		seeder.setParameters(param);
		
		FeatureFinderDefs::IndexPair peak;
		
		peak = seeder.nextSeed();
		TEST_EQUAL(peak.first,1);
		TEST_EQUAL(peak.second,0);
		
		peak = seeder.nextSeed();
		TEST_EQUAL(peak.first,1);
		TEST_EQUAL(peak.second,1);
		
		peak = seeder.nextSeed();
		TEST_EQUAL(peak.first,0);
		TEST_EQUAL(peak.second,3);
	
		TEST_EXCEPTION( FeatureFinderDefs::NoSuccessor , seeder.nextSeed() )
	}
	
	// Second test (2 unused peaks with intensity greater than 27,75)
	{
		SimpleSeeder<RawDataPoint1D,Feature> seeder(&exp, &features, &ff);
		Param param;
		param.setValue("min_intensity",0.0);
		param.setValue("intensity_perc",25.0);
		seeder.setParameters(param);
		
		FeatureFinderDefs::IndexPair peak;
		
		peak = seeder.nextSeed();
		TEST_EQUAL(peak.first,1);
		TEST_EQUAL(peak.second,2);
		
		peak = seeder.nextSeed();
		TEST_EQUAL(peak.first,0);
		TEST_EQUAL(peak.second,2);
			
		TEST_EXCEPTION( FeatureFinderDefs::NoSuccessor , seeder.nextSeed() )		
	}
	
RESULT

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
END_TEST


