// -*- Mode: C++; tab-width: 2; -*-
// vi: set ts=2:
//
// --------------------------------------------------------------------------
//                   OpenMS Mass Spectrometry Framework
// --------------------------------------------------------------------------
//  Copyright (C) 2003-2008 -- Oliver Kohlbacher, Knut Reinert
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// --------------------------------------------------------------------------
// $Maintainer: Chris Bauer$
// --------------------------------------------------------------------------

#include <OpenMS/CONCEPT/ClassTest.h>

///////////////////////////
#include <OpenMS/CHEMISTRY/TrypticIterator.h>
///////////////////////////

using namespace OpenMS;
using namespace std;

START_TEST(TrypticIterator, "$Id: TrypticIterator_test.C 2783 2008-02-28 16:18:55Z marc_sturm $")

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
typedef std::pair <String, String> FASTAEntry;

TrypticIterator* ptr = 0;
CHECK(TrypticIterator())
	ptr = new TrypticIterator();
	TEST_NOT_EQUAL(ptr, 0)
RESULT

CHECK(~TrypticIterator())
	delete ptr;
RESULT

CHECK(TrypticIterator(const TrypticIterator &))
	ptr = new TrypticIterator();
	ptr->setFastaFile("data/TrypticIterator_test.fasta");
	ptr->begin();
	++*ptr;
	TrypticIterator copy (*ptr);
	TEST_EQUAL ((*ptr).getFastaFile(),(copy).getFastaFile());
	TEST_EQUAL ((**ptr).first,(*copy).first);
	TEST_EQUAL ((**ptr).second,(*copy).second);
RESULT

CHECK(virtual void setFastaFile(const String &f) throw (Exception::FileNotFound))
	ptr = new TrypticIterator();
	TEST_EXCEPTION (Exception::FileNotFound,ptr->setFastaFile("FileThatNotExists"));
	TEST_EXCEPTION (Exception::FileNotFound,ptr->setFastaFile(""));
	ptr->setFastaFile("data/TrypticIterator_test.fasta");
RESULT

CHECK(virtual String getFastaFile())
	ptr = new TrypticIterator();
	ptr->setFastaFile("data/TrypticIterator_test.fasta");
	TEST_EQUAL(ptr->getFastaFile(),"data/TrypticIterator_test.fasta");
RESULT

CHECK(static const std::string getName())
	ptr = new TrypticIterator();
	TEST_EQUAL(ptr->getName(),"TrypticIterator");
RESULT

CHECK(static PepIterator* create())
	ptr = new TrypticIterator();
	TEST_NOT_EQUAL(ptr->create(),0);
RESULT

CHECK(virtual FASTAEntry operator *() throw (Exception::InvalidIterator))
	ptr = new TrypticIterator();
	TEST_EXCEPTION (Exception::InvalidIterator,**ptr);
	ptr->setFastaFile("data/TrypticIterator_test.fasta");
	ptr->begin();
	FASTAEntry fe = **ptr;
	TEST_EQUAL(fe.first,">Entry 1");
	TEST_EQUAL(fe.second,"AAAAAK");
	++*ptr;
	fe = **ptr;
	TEST_EQUAL(fe.first,">Entry 1");
	TEST_EQUAL(fe.second,"AAAAAKAAAAAAAAAAAAAAAAAAAAAAAA");
	++*ptr;
	fe = **ptr;
	TEST_EQUAL(fe.first,">Entry 1");
	TEST_EQUAL(fe.second,"AAAAAAAAAAAAAAAAAAAAAAAA");
	++*ptr;
	fe = **ptr;
	TEST_EQUAL(fe.first,">Entry 2");
	TEST_EQUAL(fe.second,"K");
	++*ptr;
	fe = **ptr;
	TEST_EQUAL(fe.first,">Entry 2");
	TEST_EQUAL(fe.second,"KCCCCCCCCCCCCCCCCCCCCCCCCCCCCC");
	++*ptr;
	fe = **ptr;
	TEST_EQUAL(fe.first,">Entry 2");
	TEST_EQUAL(fe.second,"CCCCCCCCCCCCCCCCCCCCCCCCCCCCC");
	++*ptr;
	fe = **ptr;
	TEST_EQUAL(fe.first,">Entry 3");
	TEST_EQUAL(fe.second,"DDDDDDDDDDDDDDDDDDDDDDDDDDDDDK");
	++*ptr;
	fe = **ptr;
	TEST_EQUAL(fe.first,">Entry 4");
	TEST_EQUAL(fe.second,"EEEEEK");
RESULT


CHECK(virtual PepIterator& operator++() throw (Exception::InvalidIterator))
	ptr = new TrypticIterator();
	TEST_EXCEPTION (Exception::InvalidIterator, ++(*ptr));
	ptr->setFastaFile("data/TrypticIterator_test.fasta");
	ptr->begin();
	PepIterator & pepIt = ++(*ptr);
	TEST_EQUAL ((*pepIt).first,(**ptr).first);
	TEST_EQUAL ((*pepIt).second,(**ptr).second);
	pepIt = ++(*ptr);
	TEST_EQUAL ((*pepIt).first,(**ptr).first);
	TEST_EQUAL ((*pepIt).second,(**ptr).second);
RESULT

CHECK(virtual PepIterator* operator++(int i) throw (Exception::InvalidIterator))
	ptr = new TrypticIterator();
	TEST_EXCEPTION (Exception::InvalidIterator, (*ptr)++);
	ptr->setFastaFile("data/TrypticIterator_test.fasta");
	ptr->begin();
	FASTAEntry fe = **ptr;
	PepIterator * pepIt = (*ptr)++;
	TEST_EQUAL ((**pepIt).first,fe.first);
	TEST_EQUAL ((**pepIt).second,fe.second);
RESULT

CHECK(virtual bool begin() throw (Exception::InvalidIterator))
	ptr = new TrypticIterator();
	TEST_EXCEPTION (Exception::InvalidIterator, (*ptr).begin());
	ptr->setFastaFile("data/TrypticIterator_test.fasta");
	ptr->begin();
	FASTAEntry fe = **ptr;
	TEST_EQUAL(fe.first,">Entry 1");
	TEST_EQUAL(fe.second,"AAAAAK");
RESULT

CHECK(virtual bool isAtEnd())
	ptr = new TrypticIterator();
	ptr->setFastaFile("data/TrypticIterator_test.fasta");
	ptr->begin();
	for (int i = 0; i < 13;i++)
	{
		TEST_EQUAL(ptr->isAtEnd(),0);
		++(*ptr);
	}
	TEST_EQUAL(ptr->isAtEnd(),1);
RESULT


CHECK(virtual void setSpectrum(const std::vector< float > &) throw (Exception::InvalidValue, Exception::NotImplemented))
	ptr = new TrypticIterator();
	const std::vector<float> spec;
	TEST_EXCEPTION (Exception::NotImplemented, (*ptr).setSpectrum(spec));
RESULT

CHECK(virtual const std::vector<float>& getSpectrum() throw (Exception::InvalidValue, Exception::NotImplemented))
	ptr = new TrypticIterator();
	TEST_EXCEPTION (Exception::NotImplemented, (*ptr).getSpectrum());
RESULT

CHECK(virtual void setTolerance(float) throw (Exception::InvalidValue, Exception::NotImplemented))
	ptr = new TrypticIterator();
	float t = 0.5;
	TEST_EXCEPTION (Exception::NotImplemented, (*ptr).setTolerance(t));
RESULT

CHECK(virtual float getTolerance() throw (Exception::InvalidValue, Exception::NotImplemented))
	ptr = new TrypticIterator();
	TEST_EXCEPTION (Exception::NotImplemented, (*ptr).getTolerance());
RESULT

CHECK(virtual bool isDigestingEnd(char aa1, char aa2))
	ptr = new TrypticIterator();
	TEST_EQUAL (ptr->isDigestingEnd('R','C'),1);
	TEST_EQUAL (ptr->isDigestingEnd('K','C'),1);
	TEST_EQUAL (ptr->isDigestingEnd('R','P'),0);
	TEST_EQUAL (ptr->isDigestingEnd('K','P'),0);
RESULT

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
END_TEST



