// -*- Mode: C++; tab-width: 2; -*-
// vi: set ts=2:
//
// --------------------------------------------------------------------------
//                   OpenMS Mass Spectrometry Framework 
// --------------------------------------------------------------------------
//  Copyright (C) 2003-2008 -- Oliver Kohlbacher, Knut Reinert
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// --------------------------------------------------------------------------
// $Maintainer: Clemens Groepl $
// --------------------------------------------------------------------------

#include <OpenMS/CONCEPT/ClassTest.h>

///////////////////////////

using namespace std;
using namespace OpenMS;

///////////////////////////

#include <OpenMS/CONCEPT/FuzzyStringComparator.h>

#include <OpenMS/DATASTRUCTURES/String.h>

/////////////////////////////////////////////////////////////

START_TEST(FuzzyStringComparator, "$Id: FuzzyStringComparator_test.C 2964 2008-03-31 09:01:57Z marc_sturm $")

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

FuzzyStringComparator* inst_ptr = 0;
CHECK((FuzzyStringComparator()))
{
	inst_ptr = new FuzzyStringComparator;
	TEST_NOT_EQUAL(inst_ptr, 0);
}
RESULT

CHECK((~FuzzyStringComparator()))
{
	delete inst_ptr;
}
RESULT

CHECK(FuzzyStringComparator& operator=(const FuzzyStringComparator& rhs))
{
  // Not implemented
	NOT_TESTABLE;
}
RESULT

CHECK(FuzzyStringComparator(const FuzzyStringComparator& rhs))
{
  // Not implemented
	NOT_TESTABLE;
}
RESULT

//------------------------------------------------------------
 
CHECK(const double& getAcceptableAbsolute() const)
{
	// tested along with set-method
	NOT_TESTABLE;
}
RESULT

CHECK(const double& getAcceptableRelative() const)
{
	// tested along with set-method
	NOT_TESTABLE;
}
RESULT

CHECK(const int& getVerboseLevel() const)
{
	// tested along with set-method
	NOT_TESTABLE;
}
RESULT

CHECK(std::ostream& getLogDestination() const)
{
	// tested along with set-method
	NOT_TESTABLE;
}
RESULT

CHECK(void setAcceptableAbsolute(const double rhs))
{
	FuzzyStringComparator fsc;
	fsc.setAcceptableAbsolute(2345.6789);
	TEST_REAL_EQUAL(fsc.getAcceptableAbsolute(),2345.6789);
}
RESULT

CHECK(void setAcceptableRelative(const double rhs))
{
	FuzzyStringComparator fsc;
	fsc.setAcceptableRelative(6789.2345);
	TEST_REAL_EQUAL(fsc.getAcceptableRelative(),6789.2345);
}
RESULT

CHECK(void setLogDestination(std::ostream & rhs))
{
	FuzzyStringComparator fsc;
	// TODO: default should be std::cout, but strangely this doesn't work with Windows, thus I commented out the first subtest. Maybe a problem with static initializers? (Clemens, 2008-03-28)
	// TEST_EQUAL(fsc.getLogDestination(),std::cout);
	fsc.setLogDestination(std::cerr);
	TEST_EQUAL(fsc.getLogDestination(),std::cerr);
	TEST_NOT_EQUAL(fsc.getLogDestination(),std::cout);
	fsc.setLogDestination(std::cout);
	TEST_NOT_EQUAL(fsc.getLogDestination(),std::cerr);
	TEST_EQUAL(fsc.getLogDestination(),std::cout);
}
RESULT

CHECK(void setVerboseLevel(const int rhs))
{
	FuzzyStringComparator fsc;
	// default should be 2
	TEST_EQUAL(fsc.getVerboseLevel(),2);
	fsc.setVerboseLevel(88);
	TEST_EQUAL(fsc.getVerboseLevel(),88);
	fsc.setVerboseLevel(-21);
	TEST_EQUAL(fsc.getVerboseLevel(),-21);
}
RESULT


//------------------------------------------------------------

CHECK((bool compare_strings( std::string const & lhs, std::string const & rhs )))
{
	std::ostringstream log;
	//------------------------------
	// A few test to show what regular expressions could not do but our class can do.
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		fsc.setVerboseLevel(2);
		fsc.setAcceptableRelative(1.00021);
		fsc.setAcceptableAbsolute(0.0);
		bool result = fsc.compare_strings("0.9999E4","1.0001E4");
		TEST_EQUAL(result,true);
		// STATUS(log.str());
	}
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		fsc.setVerboseLevel(2);
		fsc.setAcceptableRelative(1.0);
		fsc.setAcceptableAbsolute(2.0);
		bool result = fsc.compare_strings("0.9999E4","1.0001E4");
		TEST_EQUAL(result,true);
		// STATUS(log.str());
	}
	//------------------------------
	// Various issues, mixing letters, whitespace, and numbers.
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		fsc.setVerboseLevel(1);
		fsc.setAcceptableRelative(1.01);
		fsc.setAcceptableAbsolute(0.001);
		bool result = fsc.compare_strings("bl   a b 00.0022 asdfdf","bl a  b 0.00225 asdfdf");
		TEST_EQUAL(result,true);
		// STATUS(log.str());
	}
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		fsc.setVerboseLevel(1);
		fsc.setAcceptableRelative(1.01);
		fsc.setAcceptableAbsolute(0.01);
		bool result = fsc.compare_strings("bl   a 1.2   b","bl a 1.25 b");
		TEST_EQUAL(result,false);
		// STATUS(log.str());
	}
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		fsc.setVerboseLevel(1);
		fsc.setAcceptableRelative(2.);
		fsc.setAcceptableAbsolute(0.01);
		bool result = fsc.compare_strings("bl   a 1.2   b","bl a 1.25 b");
		TEST_EQUAL(result,true);
		// STATUS(log.str());
	}
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		fsc.setVerboseLevel(1);
		fsc.setAcceptableRelative(1.01);
		fsc.setAcceptableAbsolute(0.0);
		bool result = fsc.compare_strings("bl   a 1.002   b","bl a 1.0025 b");
		TEST_EQUAL(result,true);
		// STATUS(log.str());
	}
	//------------------------------
	// Test the impact of verbosity_level.
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		fsc.setVerboseLevel(1);
		fsc.setAcceptableRelative(1.03);
		fsc.setAcceptableAbsolute(0.01);
		bool result = fsc.compare_strings("1 \n 		   2	\n 3","1.01 \n \n		\n\n  					  	0002.01000 \n 3");
		TEST_EQUAL(result,true);
		std::vector<OpenMS::String> substrings;
		OpenMS::String(log.str()).split('\n',substrings);
		// STATUS(log.str());
		TEST_EQUAL(substrings.size(),0);
		STATUS(substrings.size());
	}
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		fsc.setVerboseLevel(2);
		fsc.setAcceptableRelative(1.03);
		fsc.setAcceptableAbsolute(0.01);
		bool result = fsc.compare_strings("1 \n 		   2	\n 3","1.01 \n \n		\n\n  					  	0002.01000 \n 3");
		TEST_EQUAL(result,true);
		std::vector<OpenMS::String> substrings;
		OpenMS::String(log.str()).split('\n',substrings);
		// STATUS(log.str());
		// Magic alert! - You might need to edit these numbers if report_success_() or report_failure_() changes.
		TEST_EQUAL(substrings.size(),17);
		ABORT_IF(substrings.size()!=17);
		TEST_STRING_EQUAL(substrings[0],"PASSED.");
	}
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		fsc.setVerboseLevel(1);
		fsc.setAcceptableRelative(1.01);
		fsc.setAcceptableAbsolute(0.01);
		fsc.compare_strings("1 \n 		   2	\n 3","1.11 \n \n		\n\n  					  	0004.01000 \n 3");
		std::vector<OpenMS::String> substrings;
		OpenMS::String(log.str()).split('\n',substrings);
		// STATUS(log.str());
		// Magic alert! - You might need to edit these numbers if report_success_() or report_failure_() changes.
		TEST_EQUAL(substrings.size(),31);
		ABORT_IF(substrings.size()!=31);
		TEST_STRING_EQUAL(substrings[0],"FAILED: 'ratio of numbers is too large'");
	}
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		//		fsc.setLogDestination(std::cout);
		fsc.setVerboseLevel(3);
		fsc.setAcceptableRelative(1.01);
		fsc.setAcceptableAbsolute(0.01);
		fsc.compare_strings
			(
			 "1 \n xx\n 2.008	\n 3",
			 "1.11 \nU\n		\n\n  q					  	0002.04000 \n 3"
			);
		std::vector<OpenMS::String> substrings;
		OpenMS::String(log.str()).split('\n',substrings);
		// STATUS(log.str());
		// Magic alert! - You might need to edit these numbers if report_success_() or report_failure_() changes.
		TEST_EQUAL(substrings.size(),211);
		ABORT_IF(substrings.size()!=211);
		TEST_STRING_EQUAL(substrings[0],"FAILED: 'ratio of numbers is too large'");
		TEST_STRING_EQUAL(substrings[30],"FAILED: 'input_1 is whitespace, but input_2 is not'");
		TEST_STRING_EQUAL(substrings[60],"FAILED: 'different letters'");
		TEST_STRING_EQUAL(substrings[90],"FAILED: 'line from input_2 is shorter than line from input_1'");
		TEST_STRING_EQUAL(substrings[120],"FAILED: 'input_1 is a number, but input_2 is not'");
		TEST_STRING_EQUAL(substrings[150],"FAILED: 'input_1 is not a number, but input_2 is'");
		TEST_STRING_EQUAL(substrings[180],"FAILED: 'line from input_1 is shorter than line from input_2'");
	}
}
RESULT

CHECK((bool compare_streams( std::istream & input_1, std::istream & input_2 )))
{
	std::ostringstream log;
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		fsc.setVerboseLevel(3);
		fsc.setAcceptableRelative(1.01);
		fsc.setAcceptableAbsolute(0.01);
		std::istringstream lhs("1 \n xx\n 2.008	\n 3");
		std::istringstream rhs("1.11 \nU\n		\n\n  q					  	0002.04000 \n 3");
		fsc.compare_streams(lhs,rhs);
		std::vector<OpenMS::String> substrings;
		OpenMS::String(log.str()).split('\n',substrings);
		// STATUS(log.str());
		// Magic alert! - You might need to edit these numbers if report_success_() or report_failure_() changes.
		TEST_EQUAL(substrings.size(),211);
		ABORT_IF(substrings.size()!=211);
		TEST_STRING_EQUAL(substrings[0],"FAILED: 'ratio of numbers is too large'");
		TEST_STRING_EQUAL(substrings[30],"FAILED: 'input_1 is whitespace, but input_2 is not'");
		TEST_STRING_EQUAL(substrings[60],"FAILED: 'different letters'");
		TEST_STRING_EQUAL(substrings[90],"FAILED: 'line from input_2 is shorter than line from input_1'");
		TEST_STRING_EQUAL(substrings[120],"FAILED: 'input_1 is a number, but input_2 is not'");
		TEST_STRING_EQUAL(substrings[150],"FAILED: 'input_1 is not a number, but input_2 is'");
		TEST_STRING_EQUAL(substrings[180],"FAILED: 'line from input_1 is shorter than line from input_2'");
	}
}
RESULT

CHECK((bool compare_files( const std::string & filename_1, const std::string & filename_2)))
{
	std::ostringstream log;
	{
		FuzzyStringComparator fsc;
		log.str("");
		fsc.setLogDestination(log);
		fsc.setVerboseLevel(3);
		fsc.setAcceptableRelative(1.01);
		fsc.setAcceptableAbsolute(0.01);
		std::string filename1, filename2;
		NEW_TMP_FILE(filename1);
		NEW_TMP_FILE(filename2);
		{
			std::ofstream file1(filename1.c_str());
			std::ofstream file2(filename2.c_str());
			file1 << "1 \n xx\n 2.008	\n 3" << std::flush;
			file2 << "1.11 \nU\n		\n\n  q					  	0002.04000 \n 3" << std::flush; 
			file1.close();
			file2.close();
		}
		fsc.compare_files(filename1,filename2);
		std::vector<OpenMS::String> substrings;
		OpenMS::String(log.str()).split('\n',substrings);
		// STATUS(log.str());
		// Magic alert! - You might need to edit these numbers if report_success_() or report_failure_() changes.
		TEST_EQUAL(substrings.size(),211);
		ABORT_IF(substrings.size()!=211);
		TEST_STRING_EQUAL(substrings[0],"FAILED: 'ratio of numbers is too large'");
		TEST_STRING_EQUAL(substrings[30],"FAILED: 'input_1 is whitespace, but input_2 is not'");
		TEST_STRING_EQUAL(substrings[60],"FAILED: 'different letters'");
		TEST_STRING_EQUAL(substrings[90],"FAILED: 'line from input_2 is shorter than line from input_1'");
		TEST_STRING_EQUAL(substrings[120],"FAILED: 'input_1 is a number, but input_2 is not'");
		TEST_STRING_EQUAL(substrings[150],"FAILED: 'input_1 is not a number, but input_2 is'");
		TEST_STRING_EQUAL(substrings[180],"FAILED: 'line from input_1 is shorter than line from input_2'");
	}
}
RESULT

//------------------------------------------------------------

// CHECK(void report_failure_( char const * const message ) const throw(Failure))
// {
// 	// Tested in compare_...() methods
//   NOT_TESTABLE;
// }
// RESULT

// CHECK(void report_success_() const)
// {
// 	// Tested in compare_...() methods
//   NOT_TESTABLE;
// }
// RESULT

END_TEST
